﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis.MoeaD {
  [Item("MOEADIndividual", "Represents an individual (population slot) in MOEA/D")]
  [StorableClass]
  public class MOEADIndividual<T> : Item where T : class, IItem {
    public MOEADSolution<T> Solution { get; set; }
    public double[] Weights { get; set; } // Chebyshev weights
    public List<MOEADIndividual<T>> Neighbours { get; set; } // neighbours
    public double Utility { get; set; }
    public double Fitness { get; set; }

    public MOEADIndividual(double[] weights) {
      Weights = (double[])weights.Clone();
      Neighbours = new List<MOEADIndividual<T>>();
      Utility = 1;
    }

    private class WeightComparer : IComparer<MOEADIndividual<T>> {
      private MOEADIndividual<T> individual;

      private WeightComparer() { }

      public WeightComparer(MOEADIndividual<T> individual) {
        this.individual = individual;
      }

      public int Compare(MOEADIndividual<T> x, MOEADIndividual<T> y) {
        double d1 = EuclideanDistance.GetDistance(individual.Weights, x.Weights);
        double d2 = EuclideanDistance.GetDistance(individual.Weights, y.Weights);

        return d1.CompareTo(d2);
      }
    }

    protected MOEADIndividual(MOEADIndividual<T> original, Cloner cloner) : base(original, cloner) {
      Weights = (double[])original.Weights.Clone();
      Neighbours = original.Neighbours.ToList();
      Utility = original.Utility;
      Fitness = original.Fitness;
      Solution = (MOEADSolution<T>)original.Solution.Clone(cloner);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MOEADIndividual<T>(this, cloner);
    }

    [StorableConstructor]
    protected MOEADIndividual(bool deserializing) : base(deserializing) { }
  }
}
