﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis.MoeaD {
  public interface IMOEADSolution : IItem {
    IItem Individual { get; set; }
    double[] Qualities { get; set; }
    double[] Constraints { get; set; }
    int Dimensions { get; }
  }

  [Item("MOEADSolution", "Represents a solution inside the MOEA/D population")]
  [StorableClass]
  public class MOEADSolution : Item, IMOEADSolution {
    public IItem Individual { get; set; }
    public double[] Qualities { get; set; }
    public double[] Constraints { get; set; }

    public MOEADSolution(int nObjectives, int nConstraints) {
      Qualities = new double[nObjectives];
      Constraints = new double[nConstraints];
    }

    public MOEADSolution(IItem individual, int nObjectives, int nConstraints) : this(nObjectives, nConstraints) {
      Individual = individual;
    }

    public MOEADSolution(IItem individual, double[] qualities, double[] constraints) {
      Individual = individual;
      Qualities = qualities;
      Constraints = constraints;
    }

    public MOEADSolution(double[] qualities) {
      Qualities = (double[])qualities.Clone();
    }

    public int Dimensions => Qualities == null ? 0 : Qualities.Length;

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MOEADSolution(this, cloner);
    }

    protected MOEADSolution(MOEADSolution original, Cloner cloner) : base(original, cloner) {
      Qualities = (double[])original.Qualities.Clone();
      Constraints = (double[])original.Qualities.Clone();
      Individual = (IItem)original.Individual.Clone(cloner);
    }

    [StorableConstructor]
    protected MOEADSolution(bool deserializing) : base(deserializing) { }
  }

  [Item("MOEADSolution", "Represents a solution inside the MOEA/D population")]
  [StorableClass]
  public class MOEADSolution<T> : MOEADSolution where T : class, IItem {
    public new T Individual {
      get { return (T)base.Individual; }
      set { base.Individual = value; }
    }

    public MOEADSolution(T individual, int nObjectives, int nConstraints) : base(individual, nObjectives, nConstraints) { }

    protected MOEADSolution(MOEADSolution<T> original, Cloner cloner) : base(original, cloner) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new MOEADSolution<T>(this, cloner);
    }

    [StorableConstructor]
    protected MOEADSolution(bool deserializing) : base(deserializing) { }
  }
}
